<?php
namespace BookmePro\Backend\Controllers\Coupons;

use BookmePro\Lib;

/**
 * Class Controller
 * @package BookmePro\Backend\Controllers\Coupons
 */
class Controller extends Lib\Base\Controller
{
    const page_slug = 'bookme-pro-coupons';

    /**
     * Default action
     */
    public function index()
    {
        $this->enqueueStyles(array(
            'backend' => array_merge(array(
                'bootstrap/css/bootstrap-theme.min.css',
                'css/slidePanel.min.css',
                'css/jquery.multiselect.css'
            ), (is_rtl() ? array('bootstrap/css/bootstrap-rtl.css') : array())),
            'frontend' => array('css/ladda.min.css',),
        ));

        $this->enqueueScripts(array(
            'backend' => array(
                'bootstrap/js/bootstrap.min.js' => array('jquery'),
                'js/datatables.min.js' => array('jquery'),
                'js/sidePanel.js' => array('jquery'),
                'js/jquery.multiselect.js' => array('jquery')
            ),
            'frontend' => array(
                'js/spin.min.js' => array('jquery'),
                'js/ladda.min.js' => array('jquery'),
            ),
            'module' => array('js/coupons.js' => array('jquery'))
        ));

        wp_localize_script('bookme-pro-coupons.js', 'BookmeProL10n', array(
            'csrf_token' => Lib\Utils\Common::getCsrfToken(),
            'edit' => esc_html__('Edit', 'bookme_pro'),
            'zeroRecords' => esc_html__('No coupons found.', 'bookme_pro'),
            'processing' => esc_html__('Processing...', 'bookme_pro'),
            'are_you_sure' => esc_html__('Are you sure?', 'bookme_pro'),
            'selector' => array(
                'all_selected' => esc_html__('All services', 'bookme_pro'),
                'nothing_selected' => esc_html__('No service selected', 'bookme_pro'),
                'selected' => esc_html__('selected', 'bookme_pro'),
                'collection' => Lib\Entities\Service::query()->select('id, title')->indexBy('id')->fetchArray(),
            )
        ));

        $this->render('index');
    }

    /**
     * Get coupons list
     */
    public function executeGetCoupons()
    {
        $coupons = Lib\Entities\Coupon::query('c')->select('c.*, GROUP_CONCAT(DISTINCT s.id) AS service_ids')
            ->leftJoin('CouponService', 'cs', 'cs.coupon_id = c.id')
            ->leftJoin('Service', 's', 's.id = cs.service_id')
            ->groupBy('c.id')
            ->fetchArray();
        foreach ($coupons as &$coupon) {
            $coupon['service_ids'] = $coupon['service_ids'] ? explode(',', $coupon['service_ids']) : array();
        }

        wp_send_json_success($coupons);
    }

    /**
     * Create/update coupon
     */
    public function executeSaveCoupon()
    {
        $form = new Forms\Coupon();
        $form->bind($this->getPostParameters());
        $data = $form->getData();

        if ($data['discount'] < 0 || $data['discount'] > 100) {
            wp_send_json_error(array('message' => esc_html__('Discount should be between 0 and 100.', 'bookme_pro')));
        } elseif ($data['deduction'] < 0) {
            wp_send_json_error(array('message' => esc_html__('Deduction should be a positive number.', 'bookme_pro')));
        } else {
            $coupon = $form->save();
            $service_ids = (array)$this->getParameter('service_ids');
            if (empty($service_ids)) {
                Lib\Entities\CouponService::query()
                    ->delete()
                    ->where('coupon_id', $coupon->getId())
                    ->execute();
            } else {
                Lib\Entities\CouponService::query()
                    ->delete()
                    ->where('coupon_id', $coupon->getId())
                    ->whereNotIn('service_id', $service_ids)
                    ->execute();
                $service_exists = Lib\Entities\CouponService::query()
                    ->select('service_id')
                    ->where('coupon_id', $coupon->getId())
                    ->indexBy('service_id')
                    ->fetchArray();
                foreach ($service_ids as $service_id) {
                    if (!isset($service_exists[$service_id])) {
                        $coupon_service = new Lib\Entities\CouponService();
                        $coupon_service
                            ->setCouponId($coupon->getId())
                            ->setServiceId($service_id)
                            ->save();
                    }
                }
            }

            $data = $coupon->getFields();
            $data['service_ids'] = $service_ids;
            wp_send_json_success($data);
        }
    }

    /**
     * Delete coupons.
     */
    public function executeDeleteCoupons()
    {
        $coupon_ids = array_map('intval', $this->getParameter('data', array()));
        Lib\Entities\Coupon::query()->delete()->whereIn('id', $coupon_ids)->execute();
        wp_send_json_success();
    }
}