jQuery(function ($) {
    var
        $coupons_list = $('#bookme-pro-coupons-list'),
        $check_all_button = $('#bookme-pro-check-all'),
        $coupon_modal = $('#bookme-pro-coupon-modal'),
        $coupon_new_title = $('#bookme-pro-new-coupon-title'),
        $coupon_edit_title = $('#bookme-pro-edit-coupon-title'),
        $coupon_code = $('#bookme-pro-coupon-code'),
        $coupon_discount = $('#bookme-pro-coupon-discount'),
        $coupon_deduction = $('#bookme-pro-coupon-deduction'),
        $coupon_usage_limit = $('#bookme-pro-coupon-usage-limit'),
        $save_button = $('#bookme-pro-coupon-save'),
        $add_button = $('#bookme-pro-add'),
        $delete_button = $('#bookme-pro-delete'),
        $service_list = $('.bookme-pro-js-service-selector'),
        $service_check_all = $('#bookme-pro-check-all-entities'),
        row;

    $service_list.multiselect({
        texts: {
            placeholder: $service_list.data('placeholder'), // text to use in dummy input
            selectedOptions: ' ' + $service_list.data('selected'),      // selected suffix text
            selectAll: $service_list.data('selectall'),     // select all text
            unselectAll: $service_list.data('unselectall'),   // unselect all text
            noneSelected: $service_list.data('nothing'),   // None selected text
            allSelected: $service_list.data('allselected')
        },
        showCheckbox: false,  // display the checkbox to the user
        selectAll: true, // add select all option
        minHeight: 20,
        maxPlaceholderOpts: 1
    });

    /**
     * Init DataTables.
     */
    var dt = $coupons_list.DataTable({
        order: [[0, "asc"]],
        paging: false,
        info: false,
        searching: false,
        processing: true,
        responsive: true,
        ajax: {
            url: ajaxurl,
            data: {action: 'bookme_pro_get_coupons', csrf_token: BookmeProL10n.csrf_token}
        },
        columns: [
            {data: "code"},
            {data: "discount"},
            {data: "deduction"},
            {
                data: 'service_ids',
                render: function (data, type, row, meta) {
                    if (data.length == 0) {
                        return BookmeProL10n.selector.nothing_selected;
                    } else if (data.length == 1) {
                        return BookmeProL10n.selector.collection[data[0]].title;
                    } else {
                        if (data.length == Object.keys(BookmeProL10n.selector.collection).length) {
                            return BookmeProL10n.selector.all_selected;
                        } else {
                            return data.length + ' ' + BookmeProL10n.selector.selected;
                        }
                    }
                }
            },
            {data: "usage_limit"},
            {data: "used"},
            {
                responsivePriority: 1,
                orderable: false,
                searchable: false,
                render: function (data, type, row, meta) {
                    return '<button type="button" class="btn btn-small btn-default"><i class="glyphicon glyphicon-pencil"></i></button>';
                }
            },
            {
                responsivePriority: 1,
                orderable: false,
                searchable: false,
                render: function (data, type, row, meta) {
                    return '<input type="checkbox" value="' + row.id + '">';
                }
            }
        ],
        language: {
            zeroRecords: BookmeProL10n.zeroRecords,
            processing: BookmeProL10n.processing
        }
    });

    /**
     * Select all coupons.
     */
    $check_all_button.on('change', function () {
        $coupons_list.find('tbody input:checkbox').prop('checked', this.checked);
    });

    /**
     * On coupon select.
     */
    $coupons_list.on('change', 'tbody input:checkbox', function () {
        $check_all_button.prop('checked', $coupons_list.find('tbody input:not(:checked)').length == 0);
    });

    /**
     * Edit coupon.
     */
    $coupons_list.on('click', 'button', function () {
        row = dt.row($(this).closest('td'));
        showCouponDialog();
    });

    /**
     * New coupon.
     */
    $add_button.on('click', function () {
        row = null;
        showCouponDialog();
    });


    var showCouponDialog = function () {
        var data = {};
        if (row) {
            data = row.data();
            $coupon_code.val(data.code);
            $coupon_discount.val(data.discount);
            $coupon_deduction.val(data.deduction);
            $coupon_usage_limit.val(data.usage_limit);
            $coupon_edit_title.show();
            $coupon_new_title.hide();
        } else {
            $coupon_code.val('');
            $coupon_discount.val('0');
            $coupon_deduction.val('0');
            $coupon_usage_limit.val('1');
            $coupon_edit_title.hide();
            $coupon_new_title.show();
        }
        $service_list.html('');
        $.each(BookmeProL10n.selector.collection, function (id, service) {
            $service_list.append('<option value="' + service.id + '"' + ($.inArray(service.id, data.service_ids) != -1 ? ' selected' : '') + '>' + service.title + '</option>');
        });
        $service_list.multiselect('reload');


        var event = jQuery.Event('change');
        var $checkboxes = $coupon_modal.find('input:checkbox');
        if ($checkboxes.length > 1) {
            event.target = $checkboxes[1];
        } else {
            event.target = $checkboxes[0];
        }
        $coupon_modal.trigger(event);
        showSidepanel($coupon_modal);
    };

    /**
     * Save coupon.
     */
    $save_button.on('click', function (e) {
        e.preventDefault();
        var $form = $coupon_modal.find('form');
        var data = $form.serializeArray();
        data.push({name: 'action', value: 'bookme_pro_save_coupon'});
        if (row) {
            data.push({name: 'id', value: row.data().id});
        }
        var ladda = Ladda.create(this, {timeout: 2000});
        ladda.start();
        $.ajax({
            url: ajaxurl,
            type: 'POST',
            data: data,
            dataType: 'json',
            success: function (response) {
                if (response.success) {
                    if (row) {
                        row.data(response.data).draw();
                    } else {
                        dt.row.add(response.data).draw();
                    }
                    hideSidepanel($coupon_modal);
                } else {
                    alert(response.data.message);
                }
                ladda.stop();
            }
        });
    });

    /**
     * Delete coupons.
     */
    $delete_button.on('click', function () {
        if (confirm(BookmeProL10n.are_you_sure)) {
            var ladda = Ladda.create(this);
            ladda.start();

            var data = [];
            var $checkboxes = $coupons_list.find('tbody input:checked');
            $checkboxes.each(function () {
                data.push(this.value);
            });

            $.ajax({
                url: ajaxurl,
                type: 'POST',
                data: {
                    action: 'bookme_pro_delete_coupons',
                    csrf_token: BookmeProL10n.csrf_token,
                    data: data
                },
                dataType: 'json',
                success: function (response) {
                    if (response.success) {
                        dt.rows($checkboxes.closest('td')).remove().draw();
                    } else {
                        alert(response.data.message);
                    }
                    ladda.stop();
                }
            });
        }
    });
});
