<?php
namespace BookmePro\Backend\Controllers\CustomFields;

use BookmePro\Lib;

/**
 * Class Controller
 * @package BookmePro\Backend\Controllers\CustomFields
 */
class Controller extends Lib\Base\Controller
{
    const page_slug = 'bookme-pro-custom-fields';

    /**
     *  Default Action
     */
    public function index()
    {
        $this->enqueueStyles(array(
            'frontend' => array('css/ladda.min.css', 'css/trackpad-scroll.css'),
            'backend' => array_merge(array(
                'bootstrap/css/bootstrap-theme.min.css',
                'css/tooltipster.bundle.min.css',
                'css/tooltipster-sideTip-borderless.min.css',
                'css/jquery.multiselect.css'
            ), (is_rtl() ? array('bootstrap/css/bootstrap-rtl.css') : array()))
        ));

        $this->enqueueScripts(array(
            'module' => array('js/custom_fields.js' => array('jquery-ui-sortable')),
            'frontend' => array(
                'js/spin.min.js' => array('jquery'),
                'js/ladda.min.js' => array('jquery'),
                'js/jquery.scroll.min.js' => array('jquery')
            ),
            'backend' => array(
                'bootstrap/js/bootstrap.min.js' => array('jquery'),
                'js/tooltipster.bundle.min.js' => array('jquery'),
                'js/help.js' => array('jquery'),
                'js/alert.js' => array('jquery'),
                'js/jquery.multiselect.js' => array('jquery')
            ),
        ));

        wp_localize_script('bookme-pro-custom_fields.js', 'BookmeProL10n', array(
            'csrf_token' => Lib\Utils\Common::getCsrfToken(),
            'custom_fields' => get_option('bookme_pro_custom_fields'),
            'saved' => esc_html__('Settings saved.', 'bookme_pro'),
            'selector' => array(
                'all_selected' => esc_html__('All services', 'bookme_pro'),
                'nothing_selected' => esc_html__('No service selected', 'bookme_pro'),
            ),
        ));

        $services = $this->render('_services', array('services' => Lib\Entities\Service::query()->select('id, title')->where('type', Lib\Entities\Service::TYPE_SIMPLE)->fetchArray()), false);
        $this->render('index', array('services_html' => $services));
    }

    /**
     * Save custom fields.
     */
    public function executeSaveCustomFields()
    {
        $custom_fields = $this->getParameter('fields');
        foreach (json_decode($custom_fields) as $custom_field) {
            switch ($custom_field->type) {
                case 'textarea':
                case 'text-content':
                case 'text-field':
                case 'captcha':
                    do_action('wpml_register_single_string', 'bookme_pro', 'custom_field_' . $custom_field->id . '_' . sanitize_title($custom_field->label), $custom_field->label);
                    break;
                case 'checkboxes':
                case 'radio-buttons':
                case 'drop-down':
                    do_action('wpml_register_single_string', 'bookme_pro', 'custom_field_' . $custom_field->id . '_' . sanitize_title($custom_field->label), $custom_field->label);
                    foreach ($custom_field->items as $label) {
                        do_action('wpml_register_single_string', 'bookme_pro', 'custom_field_' . $custom_field->id . '_' . sanitize_title($custom_field->label) . '=' . sanitize_title($label), $label);
                    }
                    break;
            }
        }
        update_option('bookme_pro_custom_fields', $custom_fields);
        update_option('bookme_pro_custom_fields_per_service', (int)$this->getParameter('bookme_pro_custom_fields_per_service'));
        update_option('bookme_pro_custom_fields_merge_repetitive', (int)$this->getParameter('bookme_pro_custom_fields_merge_repetitive'));
        wp_send_json_success();
    }
}