<?php if (!defined('ABSPATH')) exit; // Exit if accessed directly ?>
<form method="post" action="<?php echo esc_url(add_query_arg('tab', 'google_calendar')) ?>">
    <div class="form-group">
        <h4 class="bookme-pro-bold"><?php esc_html_e('Instructions', 'bookme_pro') ?></h4>
        <p><?php esc_html_e('To find your client ID and client secret, do the following:', 'bookme_pro') ?></p>
        <ol>
            <li><?php _e('Go to the <a href="https://console.developers.google.com/" target="_blank">Google Developers Console</a>.', 'bookme_pro') ?></li>
            <li><?php esc_html_e('Select a project, or create a new one.', 'bookme_pro') ?></li>
            <li><?php _e('Click in the upper left part to see a sliding sidebar. Next, click <b>APIs & Services</b> -> <b>Library</b>. In the list of APIs look for <b>Calendar API</b> and make sure it is enabled.', 'bookme_pro') ?></li>
            <li><?php _e('In the sidebar on the left, select <b>APIs & Services</b> -> <b>Credentials</b>.', 'bookme_pro') ?></li>
            <li><?php _e('Go to <b>OAuth consent screen</b> tab and give a name to the product, then click <b>Save</b>.', 'bookme_pro') ?></li>
            <li><?php _e('Go to <b>Credentials</b> tab and in <b>New credentials</b> drop-down menu select <b>OAuth client ID</b>.', 'bookme_pro') ?></li>
            <li><?php _e('Select <b>Web application</b> and create your project\'s OAuth 2.0 credentials by providing the necessary information. For <b>Authorized redirect URIs</b> enter the <b>Redirect URI</b> found below on this page. Click <b>Create</b>.', 'bookme_pro') ?></li>
            <li><?php _e('In the popup window look for the <b>Client ID</b> and <b>Client secret</b>. Use them in the form below on this page.', 'bookme_pro') ?></li>
            <li><?php _e('Go to Staff Members, edit a staff member and click <b>Connect</b> which is located at the bottom of the sidepanel.', 'bookme_pro') ?></li>
        </ol>
    </div>
    <?php \BookmePro\Lib\Utils\Common::optionText('bookme_pro_gc_client_id', esc_html__('Client ID', 'bookme_pro'), esc_html__('The client ID obtained from the Developers Console', 'bookme_pro')) ?>
    <?php \BookmePro\Lib\Utils\Common::optionText('bookme_pro_gc_client_secret', esc_html__('Client secret', 'bookme_pro'), esc_html__('The client secret obtained from the Developers Console', 'bookme_pro')) ?>
    <div class="form-group">
        <label for="bookme-pro-redirect-uri"><?php esc_html_e('Redirect URI', 'bookme_pro') ?></label>
        <p class="help-block"><?php esc_html_e('Enter this URL as a redirect URI in the Developers Console', 'bookme_pro') ?></p>
        <input id="bookme-pro-redirect-uri" class="form-control" type="text" readonly
               value="<?php echo \BookmePro\Lib\Google::generateRedirectURI() ?>" onclick="this.select();"
               style="cursor: pointer;"/>
    </div>
    <?php \BookmePro\Lib\Utils\Common::optionToggle('bookme_pro_gc_two_way_sync', esc_html__('2 way sync', 'bookme_pro'), esc_html__('By default Bookme Pro pushes new appointments and any further changes to Google Calendar. If you enable this option then Bookme Pro will fetch events from Google Calendar and remove corresponding time slots before displaying the second step of the booking form (this may lead to a delay when users click Next at the first step).', 'bookme_pro')) ?>
    <?php \BookmePro\Lib\Utils\Common::optionToggle('bookme_pro_gc_limit_events', esc_html__('Limit number of fetched events', 'bookme_pro'), esc_html__('If there is a lot of events in Google Calendar sometimes this leads to a lack of memory in PHP when Bookme Pro tries to fetch all events. You can limit the number of fetched events here. This only works when 2 way sync is enabled.', 'bookme_pro'),
        $values['bookme_pro_gc_limit_events']) ?>
    <?php \BookmePro\Lib\Utils\Common::optionText('bookme_pro_gc_event_title', esc_html__('Template for event title', 'bookme_pro'), esc_html__('Configure what information should be placed in the title of Google Calendar event. Available codes are {service_name}, {staff_name} and {client_names}.', 'bookme_pro')) ?>
    <div class="panel-footer">
        <?php \BookmePro\Lib\Utils\Common::csrf() ?>
        <?php \BookmePro\Lib\Utils\Common::submitButton() ?>
        <?php \BookmePro\Lib\Utils\Common::resetButton() ?>
    </div>
</form>
