<?php
namespace BookmePro\Lib;


/**
 * Class Plugin
 * @package BookmePro\Lib
 */
abstract class Plugin extends Base\Plugin
{
    protected static $prefix = 'bookme_pro_';
    protected static $title;
    protected static $version;
    protected static $slug;
    protected static $directory;
    protected static $main_file;
    protected static $basename;
    protected static $text_domain;
    protected static $root_namespace;
    protected static $embedded;

    public static function registerHooks()
    {
        parent::registerHooks();

        if (is_admin()) {
            add_action('admin_notices', function () {
                $bookme_page = isset($_REQUEST['page']) && strpos($_REQUEST['page'], 'bookme-pro-') === 0;
                if ($bookme_page) {
                    if (!(get_option('bookme_pro_secret_file') && get_option('bookme_pro_purchase_code'))) {
                        ?>
                        <div id="bookme-pro-tbs" class="wrap">
                            <div class="alert alert-info bookme-pro-tbs-body bookme-pro-flexbox">
                                <div class="bookme-pro-flex-row">
                                    <div class="bookme-pro-flex-cell" style="width:39px"><i class="alert-icon"></i>
                                    </div>
                                    <div class="bookme-pro-flex-cell">
                                        <button type="button" class="close" data-dismiss="alert">&times;</button>
                                        <span class="h4"><?php esc_html_e('Bookme Pro - License verification required.', 'bookme_pro') ?></span>
                                        <p></p>
                                        <p><?php echo sprintf(__('Please verify your license by providing a valid purchase code. <a href="%s">Click here to verify.</a>', 'bookme_pro'), Utils\Common::escAdminUrl(\BookmePro\Backend\Controllers\Settings\Controller::page_slug, array('tab' => 'purchase_code'))) ?></p>
                                    </div>
                                </div>
                            </div>
                        </div>
                        <?php
                    }
                }
            }, 10, 0);
        }

        add_action('bookme_pro_daily_task', function () {
            $staff_path = Plugin::getDirectory() . '/backend/controllers/staff/templates/';
            if ($code = get_option('bookme_pro_purchase_code')) {
                if ($result = API::check_purchase_code($code)) {
                    if (!$result['success']) {
                        if ($already_file = get_option('bookme_pro_secret_file')) {
                            if (file_exists($staff_path . $already_file . '.php')) {
                                unlink($staff_path . $already_file . '.php');
                                update_option('bookme_pro_secret_file', '');
                            }
                        }
                    }
                }
            } else {
                if ($already_file = get_option('bookme_pro_secret_file')) {
                    if (file_exists($staff_path . $already_file . '.php')) {
                        unlink($staff_path . $already_file . '.php');
                        update_option('bookme_pro_secret_file', '');
                    }
                }
            }
        }, 10, 0);
    }

    public static function run()
    {
        $dir = Plugin::getDirectory() . '/lib/addons/';
        if (is_dir($dir)) {
            foreach (glob($dir . 'bookme-pro-addon-*', GLOB_ONLYDIR) as $path) {
                include_once $path . '/init.php';
            }
        }

        parent::run();
    }

}